@@ -32,6 +32,12 @@
 #include "table.h"
 #include "table_types.h"
 
+/* current site-id in action. By default, operations are always on fallback
+ * site; We need the current site-id as global variable to avoid excessive
+ * diff of simple function parameter modifications to pass this value.
+ */
+int global_site_id = SITE_FALLBACK;
+
 /* Main program action. */
 typedef enum {
 	ACT_LIST,
@@ -64,8 +70,10 @@ struct options {
 	unsigned int no_headings:1;
 	struct util_list *base;		/* List of struct strlist_node */
 	unsigned int pairs:1;
+	unsigned int shell:1;
 	unsigned int verbose:1;
 	unsigned int quiet:1;
+	unsigned int site_id;
 };
 
 /* Makefile converts lszdev_usage.txt into C file which we include here. */
@@ -99,7 +107,9 @@ enum {
 	OPT_VERBOSE		= 'V',
 	OPT_QUIET		= 'q',
 	OPT_PAIRS		= 'P',
+	OPT_SHELL		= (OPT_ANONYMOUS_BASE+__COUNTER__),
 	OPT_AUTO_CONF		= (OPT_ANONYMOUS_BASE+__COUNTER__),
+	OPT_SITE		= 's',
 };
 
 static struct opts_conflict conflict_list[] = {
@@ -120,6 +130,8 @@ static struct opts_conflict conflict_list[] = {
 		      OPT_CONFIGURED, OPT_EXISTING, OPT_ONLINE, OPT_OFFLINE,
 		      OPT_BY_PATH, OPT_BY_NODE, OPT_BY_INTERFACE, OPT_FAILED,
 		      0),
+	OPTS_CONFLICT(OPT_SITE,
+		      OPT_TYPE),
 	OPTS_CONFLICT(OPT_ONLINE,
 		      OPT_OFFLINE),
 	OPTS_CONFLICT(OPT_QUIET,
@@ -157,13 +169,15 @@ static const struct option opt_list[] = {
 	{ "no-headings",	no_argument,	NULL, OPT_NO_HEADINGS },
 	{ "base",		required_argument, NULL, OPT_BASE },
 	{ "pairs",		no_argument,	NULL, OPT_PAIRS },
+	{ "shell",              no_argument,    NULL, OPT_SHELL },
 	{ "verbose",		no_argument,	NULL, OPT_VERBOSE },
 	{ "quiet",		no_argument,	NULL, OPT_QUIET },
+	{ "site",		required_argument, NULL, OPT_SITE },
 	{ NULL,			no_argument,	NULL, 0 },
 };
 
 /* Command line abbreviations. */
-static const char opt_str[] = ":tilLhvapPc:nVq";
+static const char opt_str[] = ":tilLhvapPc:nVqs:";
 
 /* Initialize options data structure. */
 static void init_options(struct options *opts)
@@ -172,6 +186,8 @@ static void init_options(struct options *opts)
 	opts->select = select_opts_new();
 	opts->columns = strlist_new();
 	opts->base = strlist_new();
+	/* Default operations are on fallback site*/
+	opts->site_id = SITE_FALLBACK;
 }
 
 /* Release memory used in options data structure. */
@@ -212,6 +228,41 @@ static action_t get_action(struct options *opts)
 	return ACT_LIST;
 }
 
+/* Determine the site information from the device */
+static char *get_site_from_pers(struct device *dev)
+{
+	int site, i, num = 0;
+
+	for (i = 0; i < NUM_SITES; i++) {
+		if (dev->site_specific[i].exists) {
+			site = i;
+			num++;
+			if (site == global_site_id && site != SITE_FALLBACK)
+				return misc_asprintf("s%d", site);
+		}
+	}
+
+	/* Incase of devices which do not have --site support, all the
+	 * read configurations will be in the dev->persistent and not
+	 * in dev->site_specific.
+	 */
+	if (num == 0 && dev->persistent.exists == 1 &&
+	    global_site_id == SITE_FALLBACK)
+		return misc_asprintf("yes");
+
+	/* Found more site-specific information ? return s+ */
+	if (num > 1)
+		return misc_asprintf("s+");
+
+	if (num == 1)
+		if (site == SITE_FALLBACK)
+			return misc_asprintf("yes");
+		else
+			return misc_asprintf("s%d", site);
+	else
+		return misc_asprintf("no");
+}
+
 /* Check options data structure for syntax errors. */
 static exit_code_t check_options(struct options *opts,
 				 int specified[OPTS_MAX + 1], int op)
@@ -651,6 +702,11 @@ static exit_code_t parse_options(struct options *opts, int argc, char *argv[])
 			opts->pairs = 1;
 			break;
 
+		case OPT_SHELL:
+			/* --shell */
+			opts->shell = 1;
+			break;
+
 		case OPT_VERBOSE:
 			/* --verbose */
 			opts->verbose = 1;
@@ -661,6 +717,27 @@ static exit_code_t parse_options(struct options *opts, int argc, char *argv[])
 			opts->quiet = 1;
 			break;
 
+		case OPT_SITE:
+			/* --site */
+			/* User can specify only site-ids from 0 to 9 */
+			if (!is_valid_site(optarg)) {
+				syntax("Unsupported site ID\n");
+				return EXIT_USAGE_ERROR;
+			}
+
+			if (opts->site_id != SITE_FALLBACK) {
+				syntax("Cannot specify '--site' multiple "
+				       "times\n");
+				return EXIT_USAGE_ERROR;
+			}
+
+			/* site information is a persistent configuration.
+			 * set opts->persistent here.
+			 */
+			opts->persistent = 1;
+			opts->site_id = atoi(optarg);
+			break;
+
 		case ':':
 			/* Missing option argument. */
 			syntax("Option '%s' requires an argument\n",
@@ -692,6 +769,13 @@ static exit_code_t parse_options(struct options *opts, int argc, char *argv[])
 	if (rc)
 		goto out;
 
+	/* check whether --pairs and --columns is used */
+	if (opts->shell == 1 && (opts->pairs != 1 || util_list_is_empty(opts->columns))) {
+		syntax("'--shell' must be used together with "
+		       "'--pairs' and '--columns'\n");
+		return EXIT_USAGE_ERROR;
+	}
+
 	/* Determine configuration set. */
 	if (!opts->active && !opts->persistent && !opts->auto_conf) {
 		/* Default display targets are active + persistent - note that
@@ -718,7 +802,8 @@ static exit_code_t parse_options(struct options *opts, int argc, char *argv[])
 					opts->select->all = 1;
 				else if (opts->config == config_active)
 					opts->select->existing = 1;
-				else if (opts->config == config_persistent)
+				else if (opts->config == config_persistent ||
+					 opts->site_id != SITE_FALLBACK)
 					opts->select->configured = 1;
 			}
 			break;
@@ -782,34 +867,53 @@ static struct column *dev_table = COLUMN_ARRAY(
 );
 
 static char *merge_str(const char *act, const char *pers, const char *ac,
-		       config_t config)
+		       config_t config, char **site, int max_sites)
 {
 	char *str;
-	size_t len;
+	int i;
+	struct util_list *str_list;
+
+	str_list = strlist_new();
 
 	act	= act  ? act  : "-";
 	pers	= pers ? pers : "-";
 	ac	= ac   ? ac   : "-";
 
-	if (strcmp(act, pers) == 0 && strcmp(act, ac) == 0)
+	if (strcmp(act, pers) == 0 && strcmp(act, ac) == 0 &&
+	    max_sites == 1)
 		return misc_strdup(act);
 
-	len = strlen(act) + strlen(pers) + strlen(ac) + /* 3 * / + NUL */ 4;
-	str = misc_malloc(len);
-
 	if (SCOPE_ACTIVE(config))
-		strcat(str, act);
+		strlist_add(str_list, act);
 	if (SCOPE_PERSISTENT(config)) {
-		if (*str)
-			strcat(str, "/");
-		strcat(str, pers);
+		if (global_site_id == SITE_FALLBACK) {
+			strlist_add(str_list, pers);
+		} else {
+			strlist_add(str_list, site[global_site_id]);
+			goto out;
+		}
 	}
 	if (SCOPE_AUTOCONF(config)) {
-		if (*str)
-			strcat(str, "/");
-		strcat(str, ac);
+		strlist_add(str_list, ac);
 	}
 
+	/* Do not show site-specific information when user specifically
+	 * asked for persistent configuration setting.
+	 */
+	if (SCOPE_PERSISTENT(config) && config != config_persistent) {
+		for (i = 0; i <= max_sites; i++) {
+			if (site[i])
+				strlist_add(str_list, site[i]);
+			else
+				if (str_list)
+					strlist_add(str_list, "-");
+		}
+	}
+
+out:
+	str = strlist_flatten(str_list, "/");
+	strlist_free(str_list);
+
 	return str;
 }
 
@@ -848,12 +952,13 @@ static char *dev_table_get_modules(struct device *dev)
 	return str;
 }
 
-static char *get_attr(struct device *dev, const char *name, config_t config)
+static char *get_attr(struct device *dev, const char *name, config_t config,
+		      int site_id)
 {
 	struct setting_list *list;
 	struct setting *s;
 
-	list = device_get_setting_list(dev, config);
+	list = device_get_setting_list(dev, config, site_id);
 	if (!list)
 		return NULL;
 	s = setting_list_find(list, name);
@@ -868,30 +973,61 @@ static char *get_attr(struct device *dev, const char *name, config_t config)
 	return misc_strdup(s->value);
 }
 
+/* When lszdev --info, make sure that the output contains only the
+ * valid site information. Check if the site is valid and then add
+ * the new SITEn column. For SITE_FALLBACK, the PERSISTENT column will
+ * be used.
+ */
+static bool is_site_show(int site_id, int current_site)
+{
+	if (site_id == SITE_FALLBACK || site_id == current_site)
+		return true;
+	return false;
+}
+
 /* Return string for attribute with specified @name for device @dev. */
 static char *dev_table_get_attr(struct device *dev, const char *attr,
 				config_t config)
 {
 	char *act = NULL, *pers = NULL, *ac = NULL, *str;
 	const char *name;
+	char *site[NUM_USER_SITES];
+	int i;
+	static int max_sites;
 
 	name = strchr(attr, ':');
 	if (!name)
 		return NULL;
 	name++;
 
+	/* To get the default settings on any configuration, make sure that
+	 * the site_id is specified as SITE_FALLBACK. Any value of site_id
+	 * less than SITE_FALLBACK will endup providing site-specific attribute
+	 * settings.
+	 */
 	if (SCOPE_ACTIVE(config))
-		act = get_attr(dev, name, config_active);
-	if (SCOPE_PERSISTENT(config))
-		pers = get_attr(dev, name, config_persistent);
+		act = get_attr(dev, name, config_active, SITE_FALLBACK);
+	if (SCOPE_PERSISTENT(config)) {
+		pers = get_attr(dev, name, config_persistent, SITE_FALLBACK);
+		for (i = 0; i < NUM_USER_SITES; i++)
+			site[i] = get_attr(dev, name, config, i);
+	}
 	if (SCOPE_AUTOCONF(config))
-		ac = get_attr(dev, name, config_autoconf);
+		ac = get_attr(dev, name, config_autoconf, SITE_FALLBACK);
 
-	str = merge_str(act, pers, ac, config);
+	for (i = 0; i < NUM_USER_SITES; i++)
+		if (dev->site_specific[i].exists)
+			max_sites = (max_sites < i) ? i : max_sites;
+
+	str = merge_str(act, pers, ac, config, site, max_sites);
 	free(act);
 	free(pers);
 	free(ac);
 
+	if (SCOPE_PERSISTENT(config))
+		for (i = 0; i < NUM_USER_SITES; i++)
+			free(site[i]);
+
 	return str;
 }
 
@@ -938,9 +1074,9 @@ static char *dev_table_get_value(void *item, int id, const char *heading,
 		return misc_strdup(YESNO(dev->active.exists ||
 					 dev->active.definable));
 	case dev_pers:
-		if (!dev->persistent.exists && dev->autoconf.exists)
+		if (!is_dev_pers(dev) && dev->autoconf.exists)
 			return misc_strdup("auto");
-		return misc_strdup(YESNO(dev->persistent.exists));
+		return get_site_from_pers(dev);
 	case dev_auto:
 		return misc_strdup(YESNO(dev->autoconf.exists));
 	case dev_online:
@@ -1032,11 +1168,17 @@ static struct util_list *dev_table_build(struct options *opts,
 
 		/* Only process existing devices. */
 		active = dev->active.exists || dev->active.definable;
-		persistent = dev->persistent.exists;
+		persistent = (int)is_dev_pers(dev);
 		autoconf = dev->autoconf.exists;
 		if (!active && !persistent && !autoconf)
 			continue;
 
+		/* with --site option, we consider only the persistent
+		 * configurations available on the specified site-id.
+		 * ignore active and autoconf here
+		 */
+		if (!persistent && global_site_id != SITE_FALLBACK)
+			continue;
 		ptrlist_add(devices, dev);
 	}
 
@@ -1082,7 +1224,7 @@ static exit_code_t do_list_devices(struct options *opts)
 	/* Display table. */
 	rc = table_print(dev_table, dev_table_get_value, opts, items,
 			 opts->columns, !opts->no_headings, opts->pairs, 0,
-			 util_list_is_empty(opts->columns));
+			 util_list_is_empty(opts->columns), opts->shell);
 	ptrlist_free(items, 0);
 
 	return rc;
@@ -1107,6 +1249,16 @@ enum settings_table_id {
 	settings_active,
 	settings_persistent,
 	settings_autoconf,
+	settings_site0,
+	settings_site1,
+	settings_site2,
+	settings_site3,
+	settings_site4,
+	settings_site5,
+	settings_site6,
+	settings_site7,
+	settings_site8,
+	settings_site9,
 };
 
 static struct column *settings_table = COLUMN_ARRAY(
@@ -1114,17 +1266,29 @@ static struct column *settings_table = COLUMN_ARRAY(
 	COLUMN("READONLY", align_left, settings_readonly, 1, ""),
 	COLUMN("ACTIVE", align_left, settings_active, 1, ""),
 	COLUMN("PERSISTENT", align_left, settings_persistent, 1, ""),
-	COLUMN("AUTOCONF", align_left, settings_autoconf, 0, "")
+	COLUMN("AUTOCONF", align_left, settings_autoconf, 0, ""),
+	COLUMN("SITE0", align_left, settings_site0, 0, ""),
+	COLUMN("SITE1", align_left, settings_site1, 0, ""),
+	COLUMN("SITE2", align_left, settings_site2, 0, ""),
+	COLUMN("SITE3", align_left, settings_site3, 0, ""),
+	COLUMN("SITE4", align_left, settings_site4, 0, ""),
+	COLUMN("SITE5", align_left, settings_site5, 0, ""),
+	COLUMN("SITE6", align_left, settings_site6, 0, ""),
+	COLUMN("SITE7", align_left, settings_site7, 0, ""),
+	COLUMN("SITE8", align_left, settings_site8, 0, ""),
+	COLUMN("SITE9", align_left, settings_site9, 0, "")
 );
 
 static struct util_list *settings_table_build(struct setting_list *active,
 					      struct setting_list *persistent,
 					      struct setting_list *autoconf,
+					      struct device *dev,
 					      bool readonly)
 {
 	struct util_list *names, *items;
 	struct setting *s;
 	struct strlist_node *str;
+	int i;
 
 	/* Get a list of all configured attributes. */
 	names = strlist_new();
@@ -1139,6 +1303,11 @@ static struct util_list *settings_table_build(struct setting_list *active,
 	if (persistent && !readonly) {
 		util_list_iterate(&persistent->list, s)
 			strlist_add(names, s->name);
+		if (dev) {
+			for (i = 0; i < NUM_SITES; i++)
+				util_list_iterate(&dev->site_specific[i].settings->list, s)
+					strlist_add(names, s->name);
+		}
 	}
 	if (autoconf && !readonly) {
 		util_list_iterate(&autoconf->list, s)
@@ -1159,6 +1328,7 @@ struct settings_table_data {
 	struct setting_list *active;
 	struct setting_list *persistent;
 	struct setting_list *autoconf;
+	struct setting_list *sites[NUM_SITES];
 	int pairs;
 	bool readonly;
 };
@@ -1184,6 +1354,37 @@ static char *settings_table_get_value(void *item, int id, const char *heading,
 	case settings_autoconf:
 		list = stdata->autoconf;
 		break;
+	case settings_site0:
+		list = stdata->sites[0];
+		break;
+	case settings_site1:
+		list = stdata->sites[1];
+		break;
+	case settings_site2:
+		list = stdata->sites[2];
+		break;
+	case settings_site3:
+		list = stdata->sites[3];
+		break;
+	case settings_site4:
+		list = stdata->sites[4];
+		break;
+	case settings_site5:
+		list = stdata->sites[5];
+		break;
+	case settings_site6:
+		list = stdata->sites[6];
+		break;
+	case settings_site7:
+		list = stdata->sites[7];
+		break;
+	case settings_site8:
+		list = stdata->sites[8];
+		break;
+	case settings_site9:
+		list = stdata->sites[9];
+		break;
+
 	default:
 		break;
 	}
@@ -1208,16 +1409,18 @@ static char *settings_table_get_value(void *item, int id, const char *heading,
 static void settings_table_print(struct setting_list *active,
 				 struct setting_list *persistent,
 				 struct setting_list *autoconf,
+				 struct device *dev,
 				 struct options *opts, int ind, bool readonly,
 				 bool neednl)
 {
 	struct util_list *items;
 	struct settings_table_data data;
+	int i;
 
 	items = settings_table_build(
 			SCOPE_ACTIVE(opts->config) ? active : NULL,
 			SCOPE_PERSISTENT(opts->config) ? persistent : NULL,
-			autoconf, readonly);
+			autoconf, dev, readonly);
 	if (util_list_is_empty(items)) {
 		if (!opts->pairs && !readonly)
 			indent(ind, "%sNo settings found\n",
@@ -1232,16 +1435,30 @@ static void settings_table_print(struct setting_list *active,
 	table_set_default(settings_table, settings_active,
 			  SCOPE_ACTIVE(opts->config));
 	table_set_default(settings_table, settings_persistent,
+			  is_site_show(opts->site_id, SITE_FALLBACK) &&
 			  SCOPE_PERSISTENT(opts->config) && !readonly ? 1 : 0);
 	table_set_default(settings_table, settings_autoconf,
 			  (SCOPE_AUTOCONF(opts->config) || autoconf) &&
 				!readonly ? 1 : 0);
+	if (dev) {
+		for (i = 0; i < NUM_USER_SITES; i++)
+			table_set_default(settings_table, settings_site0 + i,
+					  SCOPE_PERSISTENT(opts->config) &&
+					  is_site_show(opts->site_id, i)  &&
+					  dev->site_specific[i].exists &&
+					  !readonly ? 1 : 0);
+
+		for (i = 0; i < NUM_SITES; i++)
+			data.sites[i] = dev->site_specific[i].settings;
+	}
+
 	data.active = active;
 	data.persistent = persistent;
 	data.autoconf = autoconf;
 	data.pairs = opts->pairs;
+
 	table_print(settings_table, settings_table_get_value, &data, items,
-		    NULL, 1, opts->pairs, ind, 0);
+		    NULL, 1, opts->pairs, ind, 0, opts->shell);
 
 out:
 	ptrlist_free(items, 1);
@@ -1320,7 +1537,7 @@ static exit_code_t do_info_type(struct options *opts)
 		}
 	} else {
 		settings_table_print(dt->active_settings,
-				     dt->persistent_settings, NULL, opts,
+				     dt->persistent_settings, NULL, NULL, opts,
 				     INFO_INDENT, false, false);
 	}
 
@@ -1346,7 +1563,7 @@ static void do_list_columns(struct options *opts)
 static void do_info_one_device(struct device *dev, struct options *opts)
 {
 	struct subtype *st = dev->subtype;
-	char *names, *bdevs, *cdevs, *ndevs, *modules, *online, *path, *key;
+	char *names, *bdevs, *cdevs, *ndevs, *modules, *online, *path, *key, *sites;
 	const char *id, *type, *exists, *persist, *ac, *prefix;
 	struct util_list *resources, *errors;
 	struct strlist_node *s;
@@ -1367,6 +1584,7 @@ static void do_info_one_device(struct device *dev, struct options *opts)
 	exists	= YESNO(dev->active.exists || dev->active.definable);
 	persist	= YESNO(dev->persistent.exists);
 	ac	= YESNO(dev->autoconf.exists);
+	sites	= device_get_sites(dev);
 
 	/* Print information. */
 	if (opts->pairs) {
@@ -1384,6 +1602,9 @@ static void do_info_one_device(struct device *dev, struct options *opts)
 		print_pair("ONLINE", online);
 		print_pair("EXISTS", exists);
 		print_pair("PERSISTENT", persist);
+		if (*sites)
+			print_pair("SITES", sites);
+
 		if (SCOPE_AUTOCONF(opts->config) || dev->autoconf.exists)
 			print_pair("AUTOCONF", ac);
 		if (errors) {
@@ -1413,6 +1634,9 @@ static void do_info_one_device(struct device *dev, struct options *opts)
 		print_info("Persistent", persist);
 		if (SCOPE_AUTOCONF(opts->config) || dev->autoconf.exists)
 			print_info("Auto-configured", ac);
+		if (*sites)
+			print_info("Sites", sites);
+
 		if (errors) {
 			first = true;
 			util_list_iterate(errors, s) {
@@ -1453,17 +1677,17 @@ static void do_info_one_device(struct device *dev, struct options *opts)
 	}
 
 	settings_table_print(dev->active.exists ? dev->active.settings : NULL,
-			     dev->persistent.exists ? dev->persistent.settings :
+			     is_dev_pers(dev) ? dev->persistent.settings :
 			     NULL,
 			     dev->autoconf.exists ? dev->autoconf.settings :
-			     NULL,
+			     NULL, dev,
 			     opts, INFO_INDENT, false, !opts->pairs);
 
 	settings_table_print(dev->active.exists ? dev->active.settings : NULL,
-			     dev->persistent.exists ? dev->persistent.settings :
+			     is_dev_pers(dev) ? dev->persistent.settings :
 			     NULL,
 			     dev->autoconf.exists ? dev->autoconf.settings :
-			     NULL,
+			     NULL, dev,
 			     opts, INFO_INDENT, true, !opts->pairs);
 
 	strlist_free(errors);
@@ -1474,6 +1698,7 @@ static void do_info_one_device(struct device *dev, struct options *opts)
 	free(cdevs);
 	free(bdevs);
 	free(names);
+	free(sites);
 }
 
 /* Perform --info on devices. */
@@ -1518,10 +1743,18 @@ static exit_code_t do_info_devices(struct options *opts)
 
 		/* Only process existing devices. */
 		active = dev->active.exists || dev->active.definable;
-		persistent = dev->persistent.exists;
+		persistent = (int)is_dev_pers(dev);
 		autoconf = dev->autoconf.exists;
 		if (!active && !persistent && !autoconf)
 			continue;
+
+		/* with --site option, we consider only the persistent
+		 * configurations available on the specified site-id.
+		 * ignore active and autoconf here
+		 */
+		if (!persistent && global_site_id != SITE_FALLBACK)
+			continue;
+
 		if (found > 0)
 			printf("\n");
 		do_info_one_device(dev, opts);
@@ -1558,6 +1791,7 @@ int main(int argc, char *argv[])
 	/* Set globals. */
 	verbose		= opts.verbose;
 	quiet		= opts.quiet;
+	global_site_id	= opts.site_id;
 	path_set_base(opts.base);
 	if (opts.pairs)
 		set_stdout_data();
