@@ -3,7 +3,7 @@
  *
  * Generic input dump format functions (DFI - Dump Format Input)
  *
- * Copyright IBM Corp. 2001, 2018
+ * Copyright IBM Corp. 2001, 2023
  *
  * s390-tools is free software; you can redistribute it and/or modify
  * it under the terms of the MIT license. See LICENSE for details.
@@ -99,6 +99,15 @@ struct dfi_cpu {
 	u32		todpreg;
 	u64		vxrs_low[16];
 	struct dfi_vxrs	vxrs_high[16];
+	union {
+		u64	gscb[4];
+		struct {
+			u64 reserved;
+			u64 gsd;
+			u64 gssm;
+			u64 gs_epl_a;
+		};
+	};
 	u16		cpu_id;
 };
 
@@ -127,7 +136,8 @@ enum dfi_cpu_content {
 	DFI_CPU_CONTENT_ALL,	/* Complete register information available */
 };
 
-#define DFI_CPU_CONTENT_FAC_VX	0x00000001
+#define DFI_CPU_CONTENT_FAC_VX 0x00000001
+#define DFI_CPU_CONTENT_FAC_GS 0x00000002
 int dfi_cpu_content_fac_check(int flags);
 void dfi_cpu_content_fac_add(int flags);
 
@@ -137,6 +147,7 @@ void dfi_cpu_content_fac_add(int flags);
 struct util_list *dfi_cpu_list(void);
 void dfi_cpu_info_init(enum dfi_cpu_content content);
 struct dfi_cpu *dfi_cpu_alloc(void);
+void dfi_cpu_free(struct dfi_cpu *cpu);
 struct dfi_cpu *dfi_cpu(unsigned int cpu_nr);
 void dfi_cpu_add(struct dfi_cpu *cpu);
 unsigned int dfi_cpu_cnt(void);
@@ -238,9 +249,12 @@ extern struct dfi dfi_s390mv_ext;
 extern struct dfi dfi_s390;
 extern struct dfi dfi_s390_ext;
 extern struct dfi dfi_lkcd;
+extern struct dfi dfi_pv_elf;
 extern struct dfi dfi_elf;
 extern struct dfi dfi_kdump;
 extern struct dfi dfi_kdump_flat;
 extern struct dfi dfi_devmem;
+extern struct dfi dfi_ngdump;
+extern struct dfi dfi_vmdump;
 
 #endif /* DFI_H */
