@@ -20,12 +20,17 @@
 #include <sys/sysmacros.h>
 #include <time.h>
 #include <unistd.h>
+#include <errno.h>
 
 #include "lib/util_file.h"
 #include "lib/util_path.h"
 
 #include "zgetdump.h"
+#include "dt.h"
+#include "df_s390.h"
+#include "dfi.h"
 #include "dfi_s390mv.h"
+#include "dfi_mem_chunk.h"
 
 /*
  * Volume information
@@ -551,7 +556,6 @@ static int open_dump(void)
 	}
 	if (mv_dumper_read() != 0)
 		return -ENODEV;
-	zg_close(g.fh);
 	return 0;
 }
 
@@ -574,6 +578,8 @@ static void set_magic_numbers(void)
  */
 int dfi_s390mv_init_gen(bool extended)
 {
+	int rc;
+
 	l.extended = extended;
 	set_magic_numbers();
 	if (open_dump() != 0)
@@ -590,7 +596,9 @@ int dfi_s390mv_init_gen(bool extended)
 		return -EINVAL;
 	if (l.dump_incomplete)
 		return -EINVAL;
-	df_s390_cpu_info_add(&l.hdr, l.hdr.mem_end);
+	rc = df_s390_cpu_info_add(&l.hdr, l.hdr.mem_end);
+	if (rc)
+		return rc;
 	df_s390_em_add(&l.em);
 	return 0;
 }
