@@ -95,7 +95,7 @@ void __noreturn start(void)
 	void *load_page;
 	int config_nr;
 
-	subchannel_id = *(struct subchannel_id *)&S390_lowcore.subchannel_id;
+	subchannel_id = S390_lowcore.tpi_info.schid;
 	stage2_descr = *(struct stage2_descr*)STAGE2_DESC;
 
 	/* print menu and get configuration number */
@@ -120,22 +120,22 @@ void __noreturn start(void)
 	/* skip header */
 	entry = (struct component_entry *)
 		(load_address + sizeof(struct component_header));
-	while (entry->type == COMPONENT_LOAD ||
-	       entry->type == COMPONENT_SIGNATURE) {
-		if (entry->type == COMPONENT_SIGNATURE) {
+	while (entry->type == COMPONENT_TYPE_LOAD ||
+	       entry->type == COMPONENT_TYPE_SIGNATURE) {
+		if (entry->type == COMPONENT_TYPE_SIGNATURE) {
 			/* Skip unhandled signature components */
 			entry++;
 			continue;
 		}
-		load_address = (void *)(unsigned long)
-			entry->address.load_address[1];
+		load_address = (void *)
+			(entry->compdat.load_address & 0xfffffffful);
 		load_blocklist(entry, subchannel_id, load_address);
 		entry++;
 	}
-	if (entry->type != COMPONENT_EXECUTE)
+	if (entry->type != COMPONENT_TYPE_EXECUTE)
 		panic(EWRONGTYPE, "");
 
-	load_psw = entry->address.load_psw;
+	load_psw = entry->compdat.load_psw;
 
 	free_page((unsigned long)load_page);
 	io_irq_disable();
@@ -148,4 +148,4 @@ void panic_notify(unsigned long UNUSED(reason))
 {
 }
 
-uint64_t __section(.stage2.head) stage2_head;
+const uint64_t __section(.stage2.head) stage2_head;
