@@ -17,15 +17,7 @@
 #include "zipl.h"
 #include "boot/boot_defs.h"
 
-/*
- * For compatibility reasons zIPL can build multiple "similar"
- * program tables, which differ only in block pointers format
- */
-enum program_table_id {
-	PROGRAM_TABLE_0,
-	PROGRAM_TABLE_1,
-	NR_PROGRAM_TABLES
-};
+#define NR_PROGRAM_TABLES NR_BLKPTR_FORMATS
 
 enum program_component_id {
 	COMPONENT_ID_HEAP_AREA,
@@ -69,7 +61,6 @@ struct program_table {
 struct install_set {
 	struct program_table tables[NR_PROGRAM_TABLES];
 	struct program_component *components[NR_PROGRAM_COMPONENTS];
-	int nr_tables; /* number of "similar" program tables to be installed */
 	int nr_menu_entries;
 	int fd;
 	char *device;
@@ -77,6 +68,8 @@ struct install_set {
 	char *dump_mount_point;
 	unsigned int dump_tmp_dir_created:1;
 	unsigned int dump_mounted:1;
+	unsigned int skip_prepare:1;
+	unsigned int print_details:1;
 	struct disk_info *info;
 	disk_blockptr_t scsi_dump_sb_blockptr;
 };
