@@ -13,6 +13,8 @@
 #ifndef JOB_H
 #define JOB_H
 
+#include <stdbool.h>
+
 #include "disk.h"
 #include "zipl.h"
 
@@ -46,6 +48,8 @@ struct job_common_ipl_data {
 	address_t image_addr;
 	address_t parm_addr;
 	address_t ramdisk_addr;
+	bool optional;
+	bool ignore;
 };
 
 struct job_ipl_data {
@@ -124,6 +128,7 @@ struct job_data {
 	int dry_run;
 	int command_line;
 	int is_secure;
+	int is_ldipl_dump;
 };
 
 
@@ -131,5 +136,7 @@ int job_get(int argc, char* argv[], struct job_data** data);
 void job_free(struct job_data* job);
 int type_from_target(char *target, disk_type_t *type);
 int check_job_dump_images(struct job_dump_data* dump, char* name);
+int check_job_images_ngdump(struct job_dump_data* dump, char* name);
+bool is_ngdump_enabled(struct job_data *job);
 
 #endif /* not JOB_H */
