@@ -16,27 +16,62 @@
 #include <string.h>
 #include <fcntl.h>
 #include <sys/stat.h>
+#include "lib/util_libc.h"
+#include "boot/loaders_layout.h"
 
 #include "stage3.h"
 
-#include "../boot/data.h"
 #include "boot.h"
 #include "bootmap.h"
 #include "error.h"
 #include "misc.h"
 
-#define DATA_SIZE(x)	((size_t) (&_binary_##x##_bin_end - &_binary_##x##_bin_start))
-#define DATA_ADDR(x)	(&_binary_##x##_bin_start)
+/* Import a binary file */
+/* clang-format off */
+#define DATA_NAME(SYM, SUFFIX) _binary_##SYM##_bin##SUFFIX
+#define DATA_SIZE(SYM)	       ((size_t)(&DATA_NAME(SYM, _end) - &DATA_NAME(SYM, _start)))
+#define DATA_ADDR(SYM)	       (&DATA_NAME(SYM, _start))
+#define BIN_FILE_PATH(FILE_NAME) STRINGIFY(BUILD_PATH) "/" STRINGIFY(FILE_NAME) ".bin"
+#define IMPORT_DATA(SYM)						\
+	extern const uint8_t DATA_NAME(SYM, _start);			\
+	extern const uint8_t DATA_NAME(SYM, _end);			\
+	asm(".section \".rodata\", \"a\", @progbits\n"			\
+	    ".balign 4\n"						\
+	    ".global " STRINGIFY(DATA_NAME(SYM, _start)) "\n"		\
+	    STRINGIFY(DATA_NAME(SYM, _start)) ":\n"			\
+	    ".incbin \"" BIN_FILE_PATH(SYM) "\"\n"			\
+	    ".global " STRINGIFY(DATA_NAME(SYM, _end)) "\n"		\
+	    STRINGIFY(DATA_NAME(SYM, _end)) ":\n"			\
+	    ".balign 4\n"						\
+	    ".previous\n")
+/* clang-format on */
+
+/* Stage 0 Loader */
+IMPORT_DATA(eckd0_cdl);
+IMPORT_DATA(eckd0_ldl);
+IMPORT_DATA(fba0);
+IMPORT_DATA(tape0);
+/* Stage 1 Loader */
+IMPORT_DATA(eckd1);
+/* Stage 1b Loader */
+IMPORT_DATA(eckd1b);
+IMPORT_DATA(fba1b);
+/* Stage 2 Loader */
+IMPORT_DATA(eckd2);
+IMPORT_DATA(fba2);
+/* Stage 2 Dump Loader */
+IMPORT_DATA(eckd2dump_mv);
+IMPORT_DATA(eckd2dump_sv);
+IMPORT_DATA(fba2dump);
+IMPORT_DATA(tape2dump);
 
-#define STAGE2_MAX_SIZE		0x3000
-#define STAGE1B_LOAD_ADDR	0xe000
 #define CCW_FLAG_CC		0x40
 #define CCW_FLAG_SLI		0x20
 #define FBA_BLK_SIZE		512
 
 static struct boot_ccw0 tic_to_stage1b = {
 	.cmd = 0x08, /* tic */
-	.address_lo = STAGE1B_LOAD_ADDR,
+	.address_lo = STAGE1B_LOAD_ADDRESS,
 };
 
 /* Check sizes of internal objects. Return 0 if everything is correct,
@@ -132,8 +167,7 @@ boot_init_fba_stage0(struct boot_fba_stage0 *stage0,
 	for (i = 0; i < stage1b_count; i++) {
 		stage0->locdata[i].blocknr =
 			(uint32_t) stage1b_list[i].linear.block;
-		stage0->locread[i].read.address_lo =
-			STAGE1B_LOAD_ADDR + i * FBA_BLK_SIZE;
+		stage0->locread[i].read.address_lo = STAGE1B_LOAD_ADDRESS + i * FBA_BLK_SIZE;
 	}
 	/* Terminate CCW chain: Tic to stage 1b */
 	memcpy(&stage0->locread[i], &tic_to_stage1b, sizeof(tic_to_stage1b));
@@ -177,7 +211,7 @@ boot_init_eckd_stage1(struct boot_eckd_stage1 *stage1,
 			((stage1b_list[i].chs.cyl >> 12) & 0xfff0);
 		stage1->seek[i].sec = stage1b_list[i].chs.sec;
 		stage1->ssrt[i].read.address_lo =
-			STAGE1B_LOAD_ADDR + i * stage1b_list[i].chs.size;
+			STAGE1B_LOAD_ADDRESS + i * stage1b_list[i].chs.size;
 		stage1->ssrt[i].read.flags = CCW_FLAG_CC | CCW_FLAG_SLI;
 	}
 	/* Terminate CCW chain: Tic to stage 1b */
@@ -394,6 +428,10 @@ store_stage2_menu(void* data, size_t size, struct job_data* job)
 		return 0;
 	/* Config texts */
 	for (i = 0; i < job->data.menu.num; i++) {
+		if (job->data.menu.entry[i].data.ipl.common.ignore) {
+			params->config[job->data.menu.entry[i].pos] = 0;
+			continue;
+		}
 		const char *kdump_str = "";
 		if (job->data.menu.entry[i].data.ipl.is_kdump)
 			kdump_str = " (kdump)";
@@ -556,9 +594,12 @@ boot_get_dump_info(struct boot_info *boot_info, uint8_t dev_type, void *param)
 	       sizeof(boot_info->bp.dump.param));
 }
 
+/*
+ * Helper function to install a boot record for CCW-type IPL from DASD
+ */
 void
-boot_get_ipl_info(struct boot_info *boot_info, uint8_t dev_type,
-		  disk_blockptr_t *bm_ptr, struct disk_info *info)
+boot_get_ipl_info_ccw(struct boot_info *boot_info, uint8_t dev_type,
+		      disk_blockptr_t *bm_ptr, struct disk_info *info)
 {
 	memset(boot_info, 0, sizeof(*boot_info));
 	memcpy(&boot_info->magic, BOOT_INFO_MAGIC, sizeof(boot_info->magic));
@@ -566,6 +607,30 @@ boot_get_ipl_info(struct boot_info *boot_info, uint8_t dev_type,
 	boot_info->dev_type = dev_type;
 	boot_info->bp_type = BOOT_INFO_BP_TYPE_IPL;
 	boot_info->version = BOOT_INFO_VERSION;
-	bootmap_store_blockptr(&boot_info->bp, bm_ptr, info);
+	bootmap_store_blockptr(&boot_info->bp, bm_ptr, info,
+			       LEGACY_BLKPTR_FORMAT_ID);
 }
 
+/**
+ * Allocate and initialize a boot record for List-Directed IPL from DASD
+ *
+ * TABLE: address of an actual program table
+ */
+int boot_get_eckd_ld_ipl_br(void **br_ptr, size_t *size_ptr,
+			    disk_blockptr_t *table, struct disk_info *info)
+{
+	struct eckd_boot_record *br;
+
+	br = util_zalloc(info->phy_block_size);
+	if (!br)
+		return -1;
+
+	memcpy(&br->magic, ZIPL_MAGIC, ZIPL_MAGIC_SIZE);
+	br->version_id = DISK_LAYOUT_ID;
+	bootmap_store_blockptr(&br->program_table_pointer, table, info,
+			       BLKPTR_FORMAT_ID);
+	br->os_id = 0x55aa; /* LINUX */
+	*br_ptr = br;
+	*size_ptr = info->phy_block_size;
+	return 0;
+}
