@@ -321,7 +321,7 @@ static int add_component_file_range(struct install_set *bis,
 	size_t size;
 	int rc;
 
-	if (program_table_id)
+	if (bis->skip_prepare)
 		/* skip the preparation work */
 		goto write_segment_table;
 	if (add_files) {
@@ -402,7 +402,7 @@ static int add_component_buffer_align(struct install_set *bis, void *buffer,
 	disk_blockptr_t segment;
 	int rc;
 
-	if (program_table_id)
+	if (bis->skip_prepare)
 		/* skip the preparation work */
 		goto write_segment_table;
 	/* Write buffer */
@@ -511,9 +511,6 @@ extract_signature(const char *filename, void **ret_signature,
 	memcpy(signature, buffer + size - signature_size - sizeof(*file_sig),
 	       signature_size);
 
-	*ret_signature = signature;
-	sig_head->length = signature_size;
-
 	switch (file_sig->id_type) {
 	case PKEY_ID_PKCS7:
 		sig_head->format = PKCS7_FORMAT;
@@ -522,8 +519,12 @@ extract_signature(const char *filename, void **ret_signature,
 		error_text("Unsupported signature type %02x",
 			   file_sig->id_type);
 		signature_size = 0;
+		free(signature);
 		goto out;
 	}
+
+	sig_head->length = signature_size;
+	*ret_signature = signature;
 	/* return size of signature and corresponding header */
 	signature_size += sizeof(*file_sig);
 out:
@@ -542,13 +543,6 @@ check_remaining_filesize(size_t filesize, size_t signature_size,
 	}
 }
 
-static int is_last_table(struct install_set *bis, int table_id)
-{
-	assert(bis->nr_tables > 0 && bis->nr_tables <= NR_PROGRAM_TABLES);
-
-	return table_id == bis->nr_tables - 1;
-}
-
 static int add_ipl_program(struct install_set *bis, char *filename,
 		bool add_envblk, struct job_envblk_data *envblk,
 		struct job_ipl_data *ipl, disk_blockptr_t *program,
@@ -556,7 +550,6 @@ static int add_ipl_program(struct install_set *bis, char *filename,
 		struct job_target_data *target, int is_secure,
 		int menu_idx, int program_table_id)
 {
-	int last_table = is_last_table(bis, program_table_id);
 	struct signature_header sig_head;
 	size_t ramdisk_size, image_size;
 	size_t stage3_params_size;
@@ -588,7 +581,7 @@ static int add_ipl_program(struct install_set *bis, char *filename,
 	stats.st_size = 0;
 	if (ipl->common.ramdisk != NULL) {
 		/* Add ramdisk */
-		if (verbose && last_table)
+		if (verbose && bis->print_details)
 			printf("  initial ramdisk...: %s\n", ipl->common.ramdisk);
 		/* Get ramdisk file size */
 		if (stat(ipl->common.ramdisk, &stats)) {
@@ -646,7 +639,7 @@ static int add_ipl_program(struct install_set *bis, char *filename,
 	if (signature_size &&
 	    (is_secure == SECURE_BOOT_ENABLED ||
 	     (is_secure == SECURE_BOOT_AUTO && secure_boot_supported))) {
-		if (verbose && last_table)
+		if (verbose && bis->print_details)
 			printf("  signature for.....: %s\n", ZIPL_STAGE3_PATH);
 
 		rc = add_component_buffer(bis, signature, sig_head.length,
@@ -716,14 +709,14 @@ static int add_ipl_program(struct install_set *bis, char *filename,
 	offset += sizeof(struct component_entry);
 
 	/* Add kernel image */
-	if (verbose && last_table)
+	if (verbose && bis->print_details)
 		printf("  kernel image......: %s\n", ipl->common.image);
 
 	signature_size = extract_signature(ipl->common.image, &signature, &sig_head);
 	if (signature_size &&
 	    (is_secure == SECURE_BOOT_ENABLED ||
 	     (is_secure == SECURE_BOOT_AUTO && secure_boot_supported))) {
-		if (verbose && last_table)
+		if (verbose && bis->print_details)
 			printf("  signature for.....: %s\n", ipl->common.image);
 
 		rc = add_component_buffer(bis, signature, sig_head.length,
@@ -766,7 +759,7 @@ static int add_ipl_program(struct install_set *bis, char *filename,
 
 	/* Add kernel parmline */
 	if (ipl->common.parmline != NULL) {
-		if (verbose && last_table)
+		if (verbose && bis->print_details)
 			printf("  kernel parmline...: '%s'\n", ipl->common.parmline);
 		rc = add_component_buffer(bis, ipl->common.parmline,
 					  strlen(ipl->common.parmline) + 1,
@@ -790,7 +783,7 @@ static int add_ipl_program(struct install_set *bis, char *filename,
 		    (is_secure == SECURE_BOOT_ENABLED ||
 		     (is_secure == SECURE_BOOT_AUTO &&
 		      secure_boot_supported))) {
-			if (verbose && last_table) {
+			if (verbose && bis->print_details) {
 				printf("  signature for.....: %s\n",
 				       ipl->common.ramdisk);
 			}
@@ -883,7 +876,7 @@ static int add_ipl_program(struct install_set *bis, char *filename,
 		}
 		offset += sizeof(struct component_entry);
 	}
-	if (verbose && last_table)
+	if (verbose && bis->print_details)
 		print_components(bis, menu_idx);
 	/* Terminate component table */
 	create_component_entry(VOID_ADD(table, offset), NULL,
@@ -906,7 +899,6 @@ static int add_segment_program(struct install_set *bis,
 			       struct job_target_data *target,
 			       int program_table_id)
 {
-	int last_table = is_last_table(bis, program_table_id);
 	void *table;
 	int offset;
 	int rc;
@@ -920,7 +912,7 @@ static int add_segment_program(struct install_set *bis,
 	create_component_header(VOID_ADD(table, offset), type);
 	offset += sizeof(struct component_header);
 	/* Add segment file */
-	if (verbose && last_table)
+	if (verbose && bis->print_details)
 		printf("  segment file......: %s\n", segment->segment);
 
 	rc = add_component_file(bis, segment->segment, segment->segment_addr, 0,
@@ -935,7 +927,7 @@ static int add_segment_program(struct install_set *bis,
 	}
 	offset += sizeof(struct component_entry);
 	/* Print component addresses */
-	if (verbose && last_table)
+	if (verbose && bis->print_details)
 		print_components(bis, 0 /* menu_idx */);
 	/* Terminate component table */
 	create_component_entry(VOID_ADD(table, offset), NULL,
@@ -955,7 +947,8 @@ static int add_segment_program(struct install_set *bis,
 
 static int
 check_dump_device_late(char *partition, struct disk_info *target_info,
-		       struct job_target_data *target)
+		       struct job_target_data *target,
+		       struct job_data *job)
 {
 	struct disk_info* info;
 	int rc;
@@ -967,8 +960,10 @@ check_dump_device_late(char *partition, struct disk_info *target_info,
 			   partition);
 		return rc;
 	}
-	if ((info->type != disk_type_scsi) || (info->partnum == 0)) {
-		error_reason("Device '%s' is not a SCSI partition",
+	if ((job->is_ldipl_dump && info->type != disk_type_eckd_cdl) ||
+	    (!job->is_ldipl_dump && info->type != disk_type_scsi) ||
+	    info->partnum == 0) {
+		error_reason("Device '%s' is not a SCSI or DASD partition",
 			     partition);
 		disk_free_info(info);
 		return -1;
@@ -983,12 +978,13 @@ check_dump_device_late(char *partition, struct disk_info *target_info,
 	return 0;
 }
 
-static int add_dump_program(struct install_set *bis, struct job_dump_data *dump,
+static int add_dump_program(struct install_set *bis, struct job_data *job,
 			    disk_blockptr_t *program, int verbose,
 			    component_header_type type,
-			    struct job_target_data *target,
 			    int program_table_id)
 {
+	struct job_dump_data *dump = &job->data.dump;
+	struct job_target_data *target = &job->target;
 	struct job_ipl_data ipl;
 	int rc;
 
@@ -997,7 +993,7 @@ static int add_dump_program(struct install_set *bis, struct job_dump_data *dump,
 	ipl.common = dump->common;
 
 	/* Get file system dump parmline */
-	rc = check_dump_device_late(dump->device, bis->info, target);
+	rc = check_dump_device_late(dump->device, bis->info, target, job);
 	if (rc)
 		return rc;
 	ipl.common.parmline = dump->common.parmline;
@@ -1011,11 +1007,11 @@ static int add_dump_program(struct install_set *bis, struct job_dump_data *dump,
 /**
  * Build a program table from job data and set pointer to program table
  * block upon success
+ * PROGRAM_TABLE_ID: offset of the program table in the array (@bis->tables)
  */
 static int build_program_table(struct job_data *job,
 			       struct install_set *bis, int program_table_id)
 {
-	int last_table = is_last_table(bis, program_table_id);
 	int entries, component_header;
 	disk_blockptr_t *table;
 	int is_secure;
@@ -1033,7 +1029,7 @@ static int build_program_table(struct job_data *job,
 	/* Add programs */
 	switch (job->id) {
 	case job_ipl:
-		if (last_table) {
+		if (bis->print_details) {
 			if (job->command_line)
 				printf("Adding IPL section\n");
 			else
@@ -1052,7 +1048,7 @@ static int build_program_table(struct job_data *job,
 				     program_table_id);
 		break;
 	case job_segment:
-		if (last_table) {
+		if (bis->print_details) {
 			if (job->command_line)
 				printf("Adding segment load section\n");
 			else
@@ -1066,33 +1062,33 @@ static int build_program_table(struct job_data *job,
 		break;
 	case job_dump_partition:
 		/* Only useful for a partition dump that uses a dump kernel*/
-		if (last_table) {
+		if (bis->print_details) {
 			if (job->command_line)
 				printf("Adding dump section\n");
 			else
 				printf("Adding dump section '%s' (default)\n",
 				       job->name);
 		}
-		rc = add_dump_program(bis, &job->data.dump, &table[0],
+		rc = add_dump_program(bis, job, &table[0],
 				      verbose || job->command_line,
-				      COMPONENT_HEADER_DUMP, &job->target,
+				      COMPONENT_HEADER_DUMP,
 				      program_table_id);
 		break;
 	case job_menu:
-		if (last_table)
+		if (bis->print_details)
 			printf("Building menu '%s'\n", job->name);
 		rc = 0;
 		for (i=0; i < job->data.menu.num; i++) {
 			switch (job->data.menu.entry[i].id) {
 			case job_ipl:
-				if (last_table &&
+				if (bis->print_details &&
 				    job->data.menu.entry[i].data.ipl.common.ignore) {
 					printf("Skipping #%d: IPL section '%s' (missing files)\n",
 					       job->data.menu.entry[i].pos,
 					       job->data.menu.entry[i].name);
 					break;
 				}
-				if (last_table)
+				if (bis->print_details)
 					printf("Adding #%d: IPL section '%s'%s",
 					       job->data.menu.entry[i].pos,
 					       job->data.menu.entry[i].name,
@@ -1102,12 +1098,12 @@ static int build_program_table(struct job_data *job,
 				if (job->data.menu.entry[i].data.ipl.is_kdump) {
 					component_header =
 						COMPONENT_HEADER_DUMP;
-					if (last_table)
+					if (bis->print_details)
 						printf(" (kdump)\n");
 				} else {
 					component_header =
 						COMPONENT_HEADER_IPL;
-					if (last_table)
+					if (bis->print_details)
 						printf("\n");
 				}
 				if (job->is_secure != SECURE_BOOT_UNDEFINED)
@@ -1150,7 +1146,7 @@ static int build_program_table(struct job_data *job,
 		rc = -1;
 		break;
 	}
-	if (job->envblk.buf && verbose && last_table)
+	if (job->envblk.buf && verbose && bis->print_details)
 		envblk_print(job->envblk.buf, job->envblk.size);
 
 	if (rc == 0) {
@@ -1199,7 +1195,7 @@ static int install_stages_dasd_fba(int fd, char *filename,
 	void *stage2_data;
 
 	switch (program_table_id) {
-	case PROGRAM_TABLE_0:
+	case LEGACY_BLKPTR_FORMAT_ID:
 		/*
 		 * This program table is used for CCW-type IPL (see comments
 		 * for install_bootloader()).
@@ -1220,7 +1216,7 @@ static int install_stages_dasd_fba(int fd, char *filename,
 			return -1;
 		free(stage2_list);
 		break;
-	case PROGRAM_TABLE_1:
+	case BLKPTR_FORMAT_ID:
 		/*
 		 * This program table is not used when booting from DASD FBA
 		 * (see comments for install_bootloader()
@@ -1248,7 +1244,7 @@ static int install_stages_eckd_dasd(int fd, char *filename,
 	void *stage2b_data;
 
 	switch (program_table_id) {
-	case PROGRAM_TABLE_0:
+	case LEGACY_BLKPTR_FORMAT_ID:
 		/*
 		 * This program table is used for CCW-type IPL.
 		 * Add stage2 loader
@@ -1269,7 +1265,7 @@ static int install_stages_eckd_dasd(int fd, char *filename,
 			return -1;
 		free(stage2b_list);
 		break;
-	case PROGRAM_TABLE_1:
+	case BLKPTR_FORMAT_ID:
 		/*
 		 * This program table is used for List-Directed IPL,
 		 * which doesn't invoke stage2 loader.
@@ -1397,11 +1393,17 @@ static int disk_is_appropriate(const struct job_data *job,
 		error_reason("Secure boot forced for improper disk type");
 		return 0;
 	}
+	if (job->id == job_dump_partition &&
+	    job->is_ldipl_dump &&
+	    info->type != disk_type_eckd_cdl) {
+		error_reason("List-directed dump not support on DASD with LDL format");
+		return 0;
+	}
 	return 1;
 }
 
 static int
-check_dump_device(const struct job_data *job, const struct disk_info *info,
+check_dump_device(struct job_data *job, const struct disk_info *info,
 		  const char *device)
 {
 	int rc, part_ext;
@@ -1415,6 +1417,9 @@ check_dump_device(const struct job_data *job, const struct disk_info *info,
 	if (!disk_is_appropriate(job, info))
 		return -1;
 
+	if (is_ngdump_enabled(job))
+		return 0;
+
 	rc = util_part_search(device, info->geo.start,
 			      info->phy_blocks, info->phy_block_size, &part_ext);
 	if (rc <= 0 || part_ext) {
@@ -1430,30 +1435,15 @@ check_dump_device(const struct job_data *job, const struct disk_info *info,
 	return 0;
 }
 
-/**
- * Set actual number of "similar" program tables to be installed
- */
-static void set_nr_tables(struct job_data *job, struct install_set *bis)
-{
-	assert(bis->nr_tables == 0);
-
-	if (bis->info->type == disk_type_eckd_cdl &&
-	    (job->id == job_ipl || job->id == job_menu))
-		bis->nr_tables = NR_PROGRAM_TABLES;
-	else
-		bis->nr_tables = 1;
-}
-
 /**
  * Prepare resources to build a program table
  */
 static int prepare_build_program_table_device(struct job_data *job,
-					      struct install_set *bis,
-					      int program_table_id)
+					      struct install_set *bis)
 {
 	ulong unused_size;
 
-	if (program_table_id)
+	if (bis->skip_prepare)
 		/* skip the preparation work */
 		return 0;
 	/* Get full path of bootmap file */
@@ -1522,19 +1512,26 @@ static int prepare_build_program_table_device(struct job_data *job,
 			   bis->filename);
 		return -1;
 	}
-	set_nr_tables(job, bis);
 	return 0;
 }
 
-static int bootmap_create_device(struct job_data *job, struct install_set *bis,
-				 int program_table_id)
+/**
+ * Called when making a dump on a raw SCSI partition
+ */
+static int prepare_bootloader_device(struct job_data *job,
+				     struct install_set *bis)
 {
-	if (prepare_build_program_table_device(job, bis, program_table_id))
+	if (prepare_build_program_table_device(job, bis))
 		return -1;
-	if (build_program_table(job, bis, program_table_id))
+	/*
+	 * build a single program table at offset 1,
+	 * see comment before install_bootloader() for details
+	 */
+	bis->print_details = 1;
+	if (build_program_table(job, bis, BLKPTR_FORMAT_ID))
 		return -1;
 	/* Install stage 2 loader to bootmap if necessary */
-	if (bootmap_install_stages(job, bis, program_table_id)) {
+	if (bootmap_install_stages(job, bis, BLKPTR_FORMAT_ID)) {
 		error_text("Could not install loader stages to bootmap");
 		return -1;
 	}
@@ -1546,10 +1543,9 @@ static int bootmap_create_device(struct job_data *job, struct install_set *bis,
  */
 static int prepare_build_program_table_file(struct job_data *job,
 					    char *bootmap_dir,
-					    struct install_set *bis,
-					    int program_table_id)
+					    struct install_set *bis)
 {
-	if (program_table_id)
+	if (bis->skip_prepare)
 		/* skip the preparation work */
 		return 0;
 	/* Create temporary bootmap file */
@@ -1602,7 +1598,6 @@ static int prepare_build_program_table_file(struct job_data *job,
 		error_text("Could not write to file '%s'", bis->filename);
 		return -1;
 	}
-	set_nr_tables(job, bis);
 	return 0;
 }
 
@@ -1627,11 +1622,13 @@ static int finalize_create_file(char *bootmap_dir, struct install_set *bis)
 	return 0;
 }
 
-static int bootmap_create_file(struct job_data *job, char *bootmap_dir,
-			       struct install_set *bis, int program_table_id)
+/*
+ * PROGRAM_TABLE_ID: offset of the program table in the array (@bis->tables)
+ */
+static int bootmap_create_file(struct job_data *job, struct install_set *bis,
+			       char *bootmap_dir, int program_table_id)
 {
-	if (prepare_build_program_table_file(job, bootmap_dir, bis,
-					     program_table_id))
+	if (prepare_build_program_table_file(job, bootmap_dir, bis))
 		return -1;
 	if (build_program_table(job, bis, program_table_id))
 		return -1;
@@ -1641,8 +1638,6 @@ static int bootmap_create_file(struct job_data *job, char *bootmap_dir,
 			   bis->filename);
 		return -1;
 	}
-	if (!dry_run && is_last_table(bis, program_table_id))
-		return finalize_create_file(bootmap_dir, bis);
 	return 0;
 }
 
@@ -1680,14 +1675,12 @@ ngdump_create_meta(const char *path)
 	return 0;
 }
 
-static int bootmap_create_device_ngdump(struct job_data *job,
-					struct install_set *bis,
-					int program_table_id)
+static int prepare_bootloader_ngdump(struct job_data *job,
+				     struct install_set *bis)
 {
 	struct disk_info *info;
-	int rc;
+	char *bootmap_dir;
 
-	assert(program_table_id == 0);
 	/* Retrieve target device information */
 	if (disk_get_info(job->data.dump.device, &job->target, &info))
 		return -1;
@@ -1712,22 +1705,6 @@ static int bootmap_create_device_ngdump(struct job_data *job,
 		return -1;
 	}
 	bis->dump_tmp_dir_created = 1;
-	if (!dry_run) {
-		char *cmd = NULL;
-		util_asprintf(&cmd, "mkfs.%s -qF %s >/dev/null",
-			      NGDUMP_FSTYPE, job->data.dump.device);
-		if (verbose)
-			printf("Formatting partition '%s'\n",
-				job->data.dump.device);
-		rc = system(cmd);
-		free(cmd);
-		if (rc) {
-			error_reason(strerror(errno));
-			error_text("Could not format partition '%s':",
-				   job->data.dump.device);
-			return -1;
-		}
-	}
 	/*
 	 * Mount partition where bootmap file and also a dump file will
 	 * be stored.
@@ -1740,29 +1717,47 @@ static int bootmap_create_device_ngdump(struct job_data *job,
 		return -1;
 	}
 	bis->dump_mounted = 1;
-	if (bootmap_create_file(job, bis->dump_mount_point,
-				bis, program_table_id))
+	bootmap_dir = bis->dump_mount_point;
+	/*
+	 * Build a single program table for List-Directed IPL
+	 * See comments before install_bootloader() for details
+	 */
+	bis->print_details = 1;
+	if (bootmap_create_file(job, bis, bootmap_dir, BLKPTR_FORMAT_ID))
 		return -1;
-	if (ngdump_create_meta(bis->dump_mount_point))
+	if (!dry_run && finalize_create_file(bootmap_dir, bis))
 		return -1;
-	return 0;
+	return ngdump_create_meta(bootmap_dir);
 }
 
-
-static int
-bootmap_create(struct job_data *job, struct install_set *bis,
-	       int program_table_id)
+/**
+ * Build one or two program tables for CCW-type and(or) for List-Direceted IPL
+ * at respective offsets in the array BIS->tables. See the comment before
+ * install_bootloader() for details
+ */
+static int prepare_bootloader_ipl(struct job_data *job, struct install_set *bis)
 {
-	if (job->id == job_dump_partition) {
-		if (is_ngdump_enabled(job->data.dump.device, &job->target))
-			return bootmap_create_device_ngdump(job, bis,
-							    program_table_id);
-		else
-			return bootmap_create_device(job, bis,
-						     program_table_id);
-	} else
-		return bootmap_create_file(job, job->target.bootmap_dir,
-					   bis, program_table_id);
+	char *bootmap_dir = job->target.bootmap_dir;
+
+	/*
+	 * Build a program table for List-Directed IPL from
+	 * SCSI or ECKD DASD
+	 */
+	bis->print_details = 1;
+	if (bootmap_create_file(job, bis, bootmap_dir, BLKPTR_FORMAT_ID))
+		return -1;
+	if (bis->info->type == disk_type_scsi)
+		/* only one table to be installed per device */
+		return dry_run ? 0 : finalize_create_file(bootmap_dir, bis);
+	/*
+	 * Build one more program table for CCW-type IPL from
+	 * ECKD DASD
+	 */
+	bis->skip_prepare = 1;
+	bis->print_details = 0;
+	if (bootmap_create_file(job, bis, bootmap_dir, LEGACY_BLKPTR_FORMAT_ID))
+		return -1;
+	return dry_run ? 0 : finalize_create_file(bootmap_dir, bis);
 }
 
 /**
@@ -1796,20 +1791,19 @@ static int init_bis(struct job_data *job, struct install_set *bis)
  */
 int prepare_bootloader(struct job_data *job, struct install_set *bis)
 {
-	int i;
-	int rc;
-
 	secure_boot_supported = check_secure_boot_support();
 
-	rc = init_bis(job, bis);
-	if (rc)
-		return rc;
-	for (i = 0;; i++) {
-		rc = bootmap_create(job, bis, i);
-		if (rc || is_last_table(bis, i))
-			break;
+	if (init_bis(job, bis))
+		return -1;
+	if (job->id == job_dump_partition) {
+		if (is_ngdump_enabled(job))
+			return prepare_bootloader_ngdump(job, bis);
+		else
+			return prepare_bootloader_device(job, bis);
+	} else {
+		return prepare_bootloader_ipl(job, bis);
 	}
-	return rc;
+	return -1;
 }
 
 /**
