@@ -25,6 +25,7 @@
 
 #include "lib/zt_common.h"
 #include "lib/util_sys.h"
+#include "lib/vtoc.h"
 
 #include "boot.h"
 #include "bootmap.h"
@@ -70,9 +71,6 @@ get_scsi_layout(unsigned char* bootblock)
 	return scsi_layout_unknown;
 }
 
-
-#define DISK_LAYOUT_ID 0x00000001
-
 static int
 overwrite_partition_start(int fd, struct disk_info* info, int mv_dump_magic);
 
@@ -83,14 +81,7 @@ static int
 update_scsi_mbr(void* bootblock, disk_blockptr_t* table,
 		struct disk_info* info, disk_blockptr_t* scsi_dump_sb_blockptr)
 {
-	struct scsi_mbr {
-		uint8_t		magic[4];
-		uint32_t	version_id;
-		uint8_t		reserved[8];
-		uint8_t		program_table_pointer[16];
-		uint8_t		reserved2[0x50];
-		struct boot_info boot_info;
-	}  __packed* mbr;
+	struct scsi_mbr *mbr;
 	struct scsi_dump_param param;
 	void* buffer;
 
@@ -119,8 +110,9 @@ update_scsi_mbr(void* bootblock, disk_blockptr_t* table,
 	memset(buffer, 0, sizeof(struct scsi_mbr));
 	memcpy(&mbr->magic, ZIPL_MAGIC, ZIPL_MAGIC_SIZE);
 	mbr->version_id = DISK_LAYOUT_ID;
-	bootmap_store_blockptr(&mbr->program_table_pointer, table, info);
-	if (scsi_dump_sb_blockptr->linear.block != 0) {
+	bootmap_store_blockptr(&mbr->program_table_pointer, table, info,
+			       0 /* this argument is ignored for scsi */);
+	if (scsi_dump_sb_blockptr && scsi_dump_sb_blockptr->linear.block != 0) {
 		/* Write dump boot_info */
 		param.block = scsi_dump_sb_blockptr->linear.block *
 			      scsi_dump_sb_blockptr->linear.size;
@@ -178,11 +170,11 @@ install_scsi(int fd, disk_blockptr_t* program_table, struct disk_info* info,
 }
 
 
-/* Install bootloader for initial program load from an FBA type disk. */
+/* Install bootloader for CCW-type IPL from a FBA type disk */
 static int
-install_fba(int fd, disk_blockptr_t *program_table,
-	    disk_blockptr_t *stage1b_list, blocknum_t stage1b_count,
-	    struct disk_info *info)
+install_fba_ccw(int fd, disk_blockptr_t *program_table,
+		disk_blockptr_t *stage1b_list, blocknum_t stage1b_count,
+		struct disk_info *info)
 {
 	struct boot_fba_stage0 stage0;
 
@@ -190,8 +182,8 @@ install_fba(int fd, disk_blockptr_t *program_table,
 	if (boot_init_fba_stage0(&stage0, stage1b_list, stage1b_count))
 		return -1;
 
-	boot_get_ipl_info(&stage0.boot_info,  BOOT_INFO_DEV_TYPE_FBA,
-			  program_table, info);
+	boot_get_ipl_info_ccw(&stage0.boot_info,  BOOT_INFO_DEV_TYPE_FBA,
+			      program_table, info);
 
 	if (DRY_RUN_FUNC(misc_pwrite(fd, &stage0, sizeof(stage0), 0)))
 		return -1;
@@ -228,12 +220,14 @@ out:
 	return rc;
 }
 
-/* Install bootloader for initial program load from an ECKD type disk with
- * Linux Disk Layout. */
+/*
+ * Install bootloader for CCW-type IPL from ECKD type disk with
+ * Linux Disk Layout
+ */
 static int
-install_eckd_ldl(int fd, disk_blockptr_t *program_table,
-		 disk_blockptr_t *stage1b_list, blocknum_t stage1b_count,
-		 struct disk_info *info)
+install_eckd_ldl_ccw(int fd, disk_blockptr_t *program_table,
+		     disk_blockptr_t *stage1b_list, blocknum_t stage1b_count,
+		     struct disk_info *info)
 {
 	struct boot_eckd_ldl_stage0 stage0;
 	struct boot_eckd_stage1 stage1;
@@ -246,8 +240,8 @@ install_eckd_ldl(int fd, disk_blockptr_t *program_table,
 	if (boot_init_eckd_stage1(&stage1, stage1b_list, stage1b_count))
 		return -1;
 
-	boot_get_ipl_info(&stage1.boot_info, BOOT_INFO_DEV_TYPE_ECKD,
-			  program_table, info);
+	boot_get_ipl_info_ccw(&stage1.boot_info, BOOT_INFO_DEV_TYPE_ECKD,
+			      program_table, info);
 
 	if (DRY_RUN_FUNC(misc_pwrite(fd, &stage1, sizeof(stage1),
 				     sizeof(stage0))))
@@ -256,12 +250,14 @@ install_eckd_ldl(int fd, disk_blockptr_t *program_table,
 	return 0;
 }
 
-/* Install bootloader for initial program load from an ECKD type disk with
- * OS/390 compatible disk layout. */
-static int
-install_eckd_cdl(int fd, disk_blockptr_t *program_table,
-		 disk_blockptr_t *stage1b_list, blocknum_t stage1b_count,
-		 struct disk_info *info)
+/**
+ * Install bootloader for CCW-type IPL from ECKD type disk with
+ * OS/390 compatible disk layout
+ */
+static int install_eckd_cdl_ccw(int fd, disk_blockptr_t *program_table,
+				disk_blockptr_t *stage1b_list,
+				blocknum_t stage1b_count,
+				struct disk_info *info)
 {
 	struct boot_eckd_cdl_stage0 stage0;
 	struct boot_eckd_stage1 stage1;
@@ -274,8 +270,8 @@ install_eckd_cdl(int fd, disk_blockptr_t *program_table,
 	if (boot_init_eckd_stage1(&stage1, stage1b_list, stage1b_count))
 		return -1;
 
-	boot_get_ipl_info(&stage1.boot_info, BOOT_INFO_DEV_TYPE_ECKD,
-			  program_table, info);
+	boot_get_ipl_info_ccw(&stage1.boot_info, BOOT_INFO_DEV_TYPE_ECKD,
+			      program_table, info);
 
 	if (DRY_RUN_FUNC(misc_pwrite(fd, &stage1, sizeof(stage1),
 				     4 + info->phy_block_size)))
@@ -283,16 +279,152 @@ install_eckd_cdl(int fd, disk_blockptr_t *program_table,
 	return 0;
 }
 
-int
-install_bootloader(const char *device, disk_blockptr_t *program_table,
-		   disk_blockptr_t *scsi_dump_sb_blockptr,
-		   disk_blockptr_t *stage1b_list, blocknum_t stage1b_count,
-		   struct disk_info *info, struct job_data *job)
+/**
+ * Install a program table for List-Directed IPL on a CDL-formatted DASD.
+ *
+ * The installation means storing an actual boot record address in the
+ * volume label.
+ *
+ * BR: represents an actual boot record address.
+ */
+static int install_eckd_cdl_ld(int fd, disk_blockptr_t *br,
+			       struct disk_info *info)
 {
+	struct vol_label_cdl vl;
+	int rc;
+
+	/* Read a volume label from CDL-formatted DASD */
+	if (misc_seek(fd, 2 * info->phy_block_size))
+		return -1;
+	rc = misc_read(fd, &vl, sizeof(vl));
+	if (rc) {
+		error_text("Could not read volume label");
+		return rc;
+	}
+	/* Verify that we have a VOL1 label */
+	if (!is_vol1(vl.vollbl)) {
+		error_text("Volume label 'vol1' not initialized");
+		return -1;
+	}
+	/* Pack the actual boot record address */
+	vtoc_set_cchhb(&vl.br, br->chs.cyl, br->chs.head, br->chs.sec);
+
+	/* Write out the updated volume label */
+	if (misc_seek(fd, 2 * info->phy_block_size))
+		return -1;
+	rc = DRY_RUN_FUNC(misc_write(fd, &vl, sizeof(vl)));
+	if (rc)
+		error_text("Could not update volume lablel 'vol1'");
+	return 0;
+}
+
+/**
+ * Install program tables for CCW-type and(or) List-Directed IPL
+ * See the comment before install_bootloader() for details.
+ *
+ * TABLES: array of the program tables to be installed
+ */
+int install_bootloader_ipl(struct program_table *tables,
+			   struct disk_info *info,
+			   int fd)
+{
+	struct program_table *pt = &tables[LEGACY_BLKPTR_FORMAT_ID];
+
+	int rc = -1;
+
+	switch (info->type) {
+	case disk_type_scsi:
+		/* List-Directed IPL */
+		pt = &tables[BLKPTR_FORMAT_ID];
+		rc = install_scsi(fd, &pt->table, info, NULL);
+		break;
+	case disk_type_fba:
+		rc = install_fba_ccw(fd,
+				     &pt->table,
+				     pt->stage1b_list,
+				     pt->stage1b_count, info);
+		break;
+	case disk_type_eckd_ldl:
+		rc = install_eckd_ldl_ccw(fd,
+					  &pt->table,
+					  pt->stage1b_list,
+					  pt->stage1b_count, info);
+		break;
+	case disk_type_eckd_cdl:
+		rc = install_eckd_cdl_ccw(fd,
+					  &pt->table,
+					  pt->stage1b_list,
+					  pt->stage1b_count, info);
+		if (rc)
+			break;
+		/* install one more table for List-Directed IPL */
+		pt = &tables[BLKPTR_FORMAT_ID];
+		rc = install_eckd_cdl_ld(fd, pt->stage1b_list, info);
+		break;
+	case disk_type_diag:
+		error_reason("Inappropriarte device type (%d) for IPL",
+			     info->type);
+		break;
+	}
+	return rc;
+}
+
+/**
+ * Install a program table for List-Directed dump
+ * See the comment before install_bootloader() for details
+ */
+static int install_bootloader_dump(struct program_table *tables,
+				   struct disk_info *info,
+				   disk_blockptr_t *scsi_dump_sb_blockptr,
+				   int ngdump_enabled,
+				   int fd)
+{
+	struct program_table *pt = &tables[BLKPTR_FORMAT_ID];
+	int rc = -1;
+
+	switch (info->type) {
+	case disk_type_scsi:
+		rc = install_scsi(fd, &pt->table, info, scsi_dump_sb_blockptr);
+		if (rc == 0 && !ngdump_enabled)
+			rc = overwrite_partition_start(fd, info, 0);
+		break;
+	case disk_type_eckd_cdl:
+		rc = install_eckd_cdl_ld(fd, pt->stage1b_list, info);
+		break;
+	default:
+		error_reason("Inappropriarte device type (%d) for List-Directed dump",
+			     info->type);
+		break;
+	}
+	return rc;
+}
+
+/**
+ * Install a "compatible" boot record referring one, or two "similar"
+ * program tables.
+ *
+ * For compatibility reasons zIPL installs multiple "similar"
+ * program tables which differ only in block pointers format.
+ * Each such table is identified by an offset in the array BIS->tables
+ * of in-memory program table representations built by prepare_bootloader().
+ * The common rule is that tables built for CCW-type IPL are placed
+ * at offset 0, and tables built for List-Directed IPL are placed at
+ * offset 1. The same works for dumps.
+ */
+int install_bootloader(struct job_data *job, struct install_set *bis)
+{
+	disk_blockptr_t *scsi_dump_sb_blockptr = &bis->scsi_dump_sb_blockptr;
+	struct disk_info *info = bis->info;
+	char *device = bis->device;
 	int fd, rc;
 
+	if (!info)
+		return 0;
 	/* Inform user about what we're up to */
-	printf("Preparing boot device: ");
+	printf("Preparing boot device for %s%s: ",
+	       disk_get_ipl_type(info->type,
+				 job->id == job_dump_partition),
+	       job->id == job_dump_partition ? "dump" : "IPL");
 	if (info->name) {
 		printf("%s", info->name);
 		if (info->devno >= 0)
@@ -320,30 +452,18 @@ install_bootloader(const char *device, disk_blockptr_t *program_table,
 				"caches.\n");
 		}
 	}
-	/* Call disk specific install functions */
-	rc = -1;
-	switch (info->type) {
-	case disk_type_scsi:
-		rc = install_scsi(fd, program_table, info,
-				  scsi_dump_sb_blockptr);
-		if (rc == 0 && job->id == job_dump_partition)
-			rc = overwrite_partition_start(fd, info, 0);
-		break;
-	case disk_type_fba:
-		rc = install_fba(fd, program_table, stage1b_list,
-				 stage1b_count, info);
-		break;
-	case disk_type_eckd_ldl:
-		rc = install_eckd_ldl(fd, program_table, stage1b_list,
-				      stage1b_count, info);
-		break;
-	case disk_type_eckd_cdl:
-		rc = install_eckd_cdl(fd, program_table, stage1b_list,
-				      stage1b_count, info);
-		break;
-	case disk_type_diag:
-		/* Should not happen */
-		break;
+	/*
+	 * Depending on disk type, install one or two program tables
+	 * for CCW-type IPL and (or) for List-Directed IPL (see the
+	 * picture in comments above)
+	 */
+	if (job->id == job_dump_partition) {
+		rc = install_bootloader_dump(bis->tables, info,
+					     scsi_dump_sb_blockptr,
+					     is_ngdump_enabled(job),
+					     fd);
+	} else {
+		rc = install_bootloader_ipl(bis->tables, info, fd);
 	}
 
 	if (fsync(fd))
@@ -687,10 +807,10 @@ static int check_eckd_dump_partition(struct disk_info* info)
 static void eckd_dump_store_param(struct eckd_dump_param *param,
 				  struct disk_info *info, blocknum_t count)
 {
-	param->start_blk = info->geo.start;
-	param->end_blk = info->geo.start + info->phy_blocks - 1 - count;
+	param->blk_start = info->geo.start;
+	param->blk_end = info->geo.start + info->phy_blocks - 1 - count;
 	param->num_heads = info->geo.heads;
-	param->blocksize = info->phy_block_size;
+	param->blk_size = info->phy_block_size;
 	param->bpt = info->geo.sectors;
 }
 
@@ -901,8 +1021,8 @@ install_svdump_fba(int fd, struct disk_info *info, uint64_t mem)
 	if (boot_init_fba_stage0(&stage0, stage1b_list, stage1b_count))
 		goto out_free_stage1b_list;
 
-	param.start_blk = (uint64_t) info->geo.start;
-	param.blockct = (uint64_t) blk - 1;
+	param.blk_start = info->geo.start;
+	param.blk_end = blk - 1;
 	boot_get_dump_info(&stage0.boot_info, BOOT_INFO_DEV_TYPE_FBA, &param);
 
 	if (DRY_RUN_FUNC(misc_pwrite(fd, &stage0, sizeof(stage0), 0)))
