@@ -63,6 +63,7 @@ static struct option options[] = {
 	{ "dry-run",		no_argument,		NULL, '0'},
 	{ "force",		no_argument,		NULL, 'f'},
 	{ "kdump",		required_argument,	NULL, 'k'},
+	{ "ldipl-dump",         no_argument,            NULL, 0xb0},
 	{ "secure",		required_argument,	NULL, 'S'},
 	{ NULL,			0,			NULL, 0 }
 };
@@ -93,6 +94,7 @@ struct command_line {
 	int dry_run;
 	int force;
 	int is_secure;
+	int is_ldipl_dump;
 	enum scan_section_type type;
 };
 
@@ -284,6 +286,9 @@ get_command_line(int argc, char* argv[], struct command_line* line)
 		case 'f':
 			cmdline.force = 1;
 			break;
+		case 0xb0:
+			cmdline.is_ldipl_dump = 1;
+			break;
 		case 1:
 			/* Non-option is interpreted as section name */
 			if (cmdline.section != NULL) {
@@ -561,11 +566,13 @@ get_common_components(struct job_common_ipl_data *common,
 			    MAXIMUM_PHYSICAL_BLOCKSIZE);
 	if (rc)
 		goto error;
-	rc = set_cl_element(&cl[num++], "parmline", NULL, &common->parm_addr,
-			    MAXIMUM_PARMLINE_SIZE, 0,
-			    MAXIMUM_PHYSICAL_BLOCKSIZE);
-	if (rc)
-		goto error;
+	if (common->parmline) {
+		rc = set_cl_element(&cl[num++], "parmline", NULL, &common->parm_addr,
+				    MAXIMUM_PARMLINE_SIZE, 0,
+				    MAXIMUM_PHYSICAL_BLOCKSIZE);
+		if (rc)
+			goto error;
+	}
 	if (common->ramdisk) {
 		rc = set_cl_element(&cl[num++], "initial ramdisk",
 				    common->ramdisk, &common->ramdisk_addr,
@@ -785,6 +792,7 @@ check_common_ipl_data(struct job_common_ipl_data *common, const char *section,
 		}
 		if (size < MAX_COMMAND_LINE_SIZE + sizeof(uint64_t)) {
 			error_text_section("Image file", section, common->image);
+			free(buffer);
 			return -1;
 		}
 
@@ -797,8 +805,11 @@ check_common_ipl_data(struct job_common_ipl_data *common, const char *section,
 			error_text("The length of the parameters line "
 				   "(%d bytes) exceeds the allowed maximum "
 				   "(%d bytes) in section '%s'", len, max_parm_size, section);
+			free(buffer);
 			return -1;
 		}
+		free(buffer);
+		buffer = NULL;
 	}
 skip_image:
 	if (common->ramdisk != NULL) {
@@ -908,7 +919,7 @@ check_job_images_ngdump(struct job_dump_data* dump, char *name)
 	FILE *fp;
 	int rc;
 
-	misc_asprintf(&ppn_cmd, "%s %s", helper, dump->device);
+	misc_asprintf(&ppn_cmd, "%s %s %d", helper, dump->device, dry_run);
 	printf("Run %s\n", ppn_cmd);
 
 	fp = popen(ppn_cmd, "r");
@@ -1972,6 +1983,7 @@ job_get(int argc, char* argv[], struct job_data** data)
 	job->data.mvdump.force = cmdline.force;
 	job->dry_run = cmdline.dry_run;
 	job->is_secure =  SECURE_BOOT_UNDEFINED;
+	job->is_ldipl_dump = cmdline.is_ldipl_dump;
 	if (job->verbose)
 		printf("Looking for components in '%s'\n", util_libdir());
 
@@ -2014,7 +2026,9 @@ job_get(int argc, char* argv[], struct job_data** data)
 }
 
 bool
-is_ngdump_enabled(const char* device, struct job_target_data* target)
+is_ngdump_enabled(struct job_data *job)
 {
-	return disk_is_nvme(device, target);
+	if (job->is_ldipl_dump)
+		return true;
+	return disk_is_nvme(job->data.dump.device, &job->target);
 }
